# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_RESAMPLE_METADATA = Metadata(
    id="bc1ecda25e27333d33b361ed48fef4f0c37935a7.workbench",
    name="volume-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeResampleFlirtParamsDictNoTag = typing.TypedDict('_VolumeResampleFlirtParamsDictNoTag', {
    "source-volume": str,
    "target-volume": str,
})
VolumeResampleFlirtParamsDictTagged = typing.TypedDict('VolumeResampleFlirtParamsDictTagged', {
    "@type": typing.Literal["flirt"],
    "source-volume": str,
    "target-volume": str,
})
VolumeResampleFlirtParamsDict = _VolumeResampleFlirtParamsDictNoTag | VolumeResampleFlirtParamsDictTagged


_VolumeResampleAffineParamsDictNoTag = typing.TypedDict('_VolumeResampleAffineParamsDictNoTag', {
    "affine": str,
    "flirt": typing.NotRequired[VolumeResampleFlirtParamsDict | None],
})
VolumeResampleAffineParamsDictTagged = typing.TypedDict('VolumeResampleAffineParamsDictTagged', {
    "@type": typing.Literal["affine"],
    "affine": str,
    "flirt": typing.NotRequired[VolumeResampleFlirtParamsDict | None],
})
VolumeResampleAffineParamsDict = _VolumeResampleAffineParamsDictNoTag | VolumeResampleAffineParamsDictTagged


_VolumeResampleFlirtParamsDict_NoTag = typing.TypedDict('_VolumeResampleFlirtParamsDict_NoTag', {
    "source-volume": str,
    "target-volume": str,
})
VolumeResampleFlirtParamsDictTagged_ = typing.TypedDict('VolumeResampleFlirtParamsDictTagged_', {
    "@type": typing.Literal["flirt"],
    "source-volume": str,
    "target-volume": str,
})
VolumeResampleFlirtParamsDict_ = _VolumeResampleFlirtParamsDict_NoTag | VolumeResampleFlirtParamsDictTagged_


_VolumeResampleAffineSeriesParamsDictNoTag = typing.TypedDict('_VolumeResampleAffineSeriesParamsDictNoTag', {
    "affine-series": str,
    "flirt": typing.NotRequired[VolumeResampleFlirtParamsDict_ | None],
})
VolumeResampleAffineSeriesParamsDictTagged = typing.TypedDict('VolumeResampleAffineSeriesParamsDictTagged', {
    "@type": typing.Literal["affine-series"],
    "affine-series": str,
    "flirt": typing.NotRequired[VolumeResampleFlirtParamsDict_ | None],
})
VolumeResampleAffineSeriesParamsDict = _VolumeResampleAffineSeriesParamsDictNoTag | VolumeResampleAffineSeriesParamsDictTagged


_VolumeResampleWarpParamsDictNoTag = typing.TypedDict('_VolumeResampleWarpParamsDictNoTag', {
    "warpfield": str,
    "source-volume": typing.NotRequired[str | None],
})
VolumeResampleWarpParamsDictTagged = typing.TypedDict('VolumeResampleWarpParamsDictTagged', {
    "@type": typing.Literal["warp"],
    "warpfield": str,
    "source-volume": typing.NotRequired[str | None],
})
VolumeResampleWarpParamsDict = _VolumeResampleWarpParamsDictNoTag | VolumeResampleWarpParamsDictTagged


_VolumeResampleParamsDictNoTag = typing.TypedDict('_VolumeResampleParamsDictNoTag', {
    "volume-in": InputPathType,
    "volume-space": str,
    "method": str,
    "volume-out": str,
    "affine": typing.NotRequired[list[VolumeResampleAffineParamsDict] | None],
    "affine-series": typing.NotRequired[list[VolumeResampleAffineSeriesParamsDict] | None],
    "warp": typing.NotRequired[list[VolumeResampleWarpParamsDict] | None],
    "value": typing.NotRequired[float | None],
})
VolumeResampleParamsDictTagged = typing.TypedDict('VolumeResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-resample"],
    "volume-in": InputPathType,
    "volume-space": str,
    "method": str,
    "volume-out": str,
    "affine": typing.NotRequired[list[VolumeResampleAffineParamsDict] | None],
    "affine-series": typing.NotRequired[list[VolumeResampleAffineSeriesParamsDict] | None],
    "warp": typing.NotRequired[list[VolumeResampleWarpParamsDict] | None],
    "value": typing.NotRequired[float | None],
})
VolumeResampleParamsDict = _VolumeResampleParamsDictNoTag | VolumeResampleParamsDictTagged


def volume_resample_flirt(
    source_volume: str,
    target_volume: str,
) -> VolumeResampleFlirtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source_volume: the source volume used when generating the affine.
        target_volume: the target volume used when generating the affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt",
        "source-volume": source_volume,
        "target-volume": target_volume,
    }
    return params


def volume_resample_flirt_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleFlirtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("target-volume", None) is None:
        raise StyxValidationError("`target-volume` must not be None")
    if not isinstance(params["target-volume"], str):
        raise StyxValidationError(f'`target-volume` has the wrong type: Received `{type(params.get("target-volume", None))}` expected `str`')


def volume_resample_flirt_cargs(
    params: VolumeResampleFlirtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-flirt",
        params.get("source-volume", None),
        params.get("target-volume", None)
    ])
    return cargs


def volume_resample_affine(
    affine: str,
    flirt: VolumeResampleFlirtParamsDict | None = None,
) -> VolumeResampleAffineParamsDictTagged:
    """
    Build parameters.
    
    Args:
        affine: the affine file to use.
        flirt: MUST be used if affine is a flirt affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "affine",
        "affine": affine,
    }
    if flirt is not None:
        params["flirt"] = flirt
    return params


def volume_resample_affine_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleAffineParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("affine", None) is None:
        raise StyxValidationError("`affine` must not be None")
    if not isinstance(params["affine"], str):
        raise StyxValidationError(f'`affine` has the wrong type: Received `{type(params.get("affine", None))}` expected `str`')
    if params.get("flirt", None) is not None:
        volume_resample_flirt_validate(params["flirt"])


def volume_resample_affine_cargs(
    params: VolumeResampleAffineParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-affine",
        params.get("affine", None),
        *(volume_resample_flirt_cargs(params.get("flirt", None), execution) if (params.get("flirt", None) is not None) else [])
    ])
    return cargs


def volume_resample_flirt_(
    source_volume: str,
    target_volume: str,
) -> VolumeResampleFlirtParamsDictTagged_:
    """
    Build parameters.
    
    Args:
        source_volume: the source volume used when generating the affine.
        target_volume: the target volume used when generating the affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt",
        "source-volume": source_volume,
        "target-volume": target_volume,
    }
    return params


def volume_resample_flirt_validate_(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleFlirtParamsDict_` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("target-volume", None) is None:
        raise StyxValidationError("`target-volume` must not be None")
    if not isinstance(params["target-volume"], str):
        raise StyxValidationError(f'`target-volume` has the wrong type: Received `{type(params.get("target-volume", None))}` expected `str`')


def volume_resample_flirt_cargs_(
    params: VolumeResampleFlirtParamsDict_,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-flirt",
        params.get("source-volume", None),
        params.get("target-volume", None)
    ])
    return cargs


def volume_resample_affine_series(
    affine_series: str,
    flirt: VolumeResampleFlirtParamsDict_ | None = None,
) -> VolumeResampleAffineSeriesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        affine_series: text file containing 12 or 16 numbers per line, each\
            being a row-major flattened affine.
        flirt: MUST be used if the affines are flirt affines.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "affine-series",
        "affine-series": affine_series,
    }
    if flirt is not None:
        params["flirt"] = flirt
    return params


def volume_resample_affine_series_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleAffineSeriesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("affine-series", None) is None:
        raise StyxValidationError("`affine-series` must not be None")
    if not isinstance(params["affine-series"], str):
        raise StyxValidationError(f'`affine-series` has the wrong type: Received `{type(params.get("affine-series", None))}` expected `str`')
    if params.get("flirt", None) is not None:
        volume_resample_flirt_validate_(params["flirt"])


def volume_resample_affine_series_cargs(
    params: VolumeResampleAffineSeriesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-affine-series",
        params.get("affine-series", None),
        *(volume_resample_flirt_cargs_(params.get("flirt", None), execution) if (params.get("flirt", None) is not None) else [])
    ])
    return cargs


def volume_resample_warp(
    warpfield: str,
    source_volume: str | None = None,
) -> VolumeResampleWarpParamsDictTagged:
    """
    Build parameters.
    
    Args:
        warpfield: the warpfield file.
        source_volume: MUST be used if using a fnirt warpfield\
            \
            the source volume used when generating the warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "warp",
        "warpfield": warpfield,
    }
    if source_volume is not None:
        params["source-volume"] = source_volume
    return params


def volume_resample_warp_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleWarpParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("warpfield", None) is None:
        raise StyxValidationError("`warpfield` must not be None")
    if not isinstance(params["warpfield"], str):
        raise StyxValidationError(f'`warpfield` has the wrong type: Received `{type(params.get("warpfield", None))}` expected `str`')
    if params.get("source-volume", None) is not None:
        if not isinstance(params["source-volume"], str):
            raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str | None`')


def volume_resample_warp_cargs(
    params: VolumeResampleWarpParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-warp",
        params.get("warpfield", None)
    ])
    if params.get("source-volume", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("source-volume", None)
        ])
    return cargs


class VolumeResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_resample_params(
    volume_in: InputPathType,
    volume_space: str,
    method: str,
    volume_out: str,
    affine: list[VolumeResampleAffineParamsDict] | None = None,
    affine_series: list[VolumeResampleAffineSeriesParamsDict] | None = None,
    warp: list[VolumeResampleWarpParamsDict] | None = None,
    value: float | None = None,
) -> VolumeResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: volume to resample.
        volume_space: a volume file in the volume space you want for the output.
        method: the resampling method.
        volume_out: the output volume.
        affine: add an affine transform.
        affine_series: add an independent affine per-frame.
        warp: add a nonlinear warpfield transform.
        value: use a specified value for locations outside the FoV of the input\
            image or warpfield(s)\
            \
            the value to use (default 0).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-resample",
        "volume-in": volume_in,
        "volume-space": volume_space,
        "method": method,
        "volume-out": volume_out,
    }
    if affine is not None:
        params["affine"] = affine
    if affine_series is not None:
        params["affine-series"] = affine_series
    if warp is not None:
        params["warp"] = warp
    if value is not None:
        params["value"] = value
    return params


def volume_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("volume-space", None) is None:
        raise StyxValidationError("`volume-space` must not be None")
    if not isinstance(params["volume-space"], str):
        raise StyxValidationError(f'`volume-space` has the wrong type: Received `{type(params.get("volume-space", None))}` expected `str`')
    if params.get("method", None) is None:
        raise StyxValidationError("`method` must not be None")
    if not isinstance(params["method"], str):
        raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("affine", None) is not None:
        if not isinstance(params["affine"], list):
            raise StyxValidationError(f'`affine` has the wrong type: Received `{type(params.get("affine", None))}` expected `list[VolumeResampleAffineParamsDict] | None`')
        for e in params["affine"]:
            volume_resample_affine_validate(e)
    if params.get("affine-series", None) is not None:
        if not isinstance(params["affine-series"], list):
            raise StyxValidationError(f'`affine-series` has the wrong type: Received `{type(params.get("affine-series", None))}` expected `list[VolumeResampleAffineSeriesParamsDict] | None`')
        for e in params["affine-series"]:
            volume_resample_affine_series_validate(e)
    if params.get("warp", None) is not None:
        if not isinstance(params["warp"], list):
            raise StyxValidationError(f'`warp` has the wrong type: Received `{type(params.get("warp", None))}` expected `list[VolumeResampleWarpParamsDict] | None`')
        for e in params["warp"]:
            volume_resample_warp_validate(e)
    if params.get("value", None) is not None:
        if not isinstance(params["value"], (float, int)):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float | None`')


def volume_resample_cargs(
    params: VolumeResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-resample"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("volume-space", None))
    cargs.append(params.get("method", None))
    cargs.append(params.get("volume-out", None))
    if params.get("affine", None) is not None or params.get("affine-series", None) is not None or params.get("warp", None) is not None:
        cargs.extend([
            *([a for c in [volume_resample_affine_cargs(s, execution) for s in params.get("affine", None)] for a in c] if (params.get("affine", None) is not None) else []),
            *([a for c in [volume_resample_affine_series_cargs(s, execution) for s in params.get("affine-series", None)] for a in c] if (params.get("affine-series", None) is not None) else []),
            *([a for c in [volume_resample_warp_cargs(s, execution) for s in params.get("warp", None)] for a in c] if (params.get("warp", None) is not None) else [])
        ])
    if params.get("value", None) is not None:
        cargs.extend([
            "-background",
            str(params.get("value", None))
        ])
    return cargs


def volume_resample_outputs(
    params: VolumeResampleParamsDict,
    execution: Execution,
) -> VolumeResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeResampleOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_resample_execute(
    params: VolumeResampleParamsDict,
    runner: Runner | None = None,
) -> VolumeResampleOutputs:
    """
    TRANSFORM AND RESAMPLE A VOLUME FILE.
    
    Resample a volume file with an arbitrary list of transformations. You may
    specify -affine, -warp, and -affine-series multiple times each, and they
    will be used in the order specified. For instance, for rigid motion
    correction followed by nonlinear atlas registration, specify -affine-series
    first, then -warp. The recommended methods are CUBIC (cubic spline) for most
    data, and ENCLOSING_VOXEL for label data. The parameter <method> must be one
    of:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeResampleOutputs`).
    """
    volume_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = volume_resample_cargs(params, execution)
    ret = volume_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_resample(
    volume_in: InputPathType,
    volume_space: str,
    method: str,
    volume_out: str,
    affine: list[VolumeResampleAffineParamsDict] | None = None,
    affine_series: list[VolumeResampleAffineSeriesParamsDict] | None = None,
    warp: list[VolumeResampleWarpParamsDict] | None = None,
    value: float | None = None,
    runner: Runner | None = None,
) -> VolumeResampleOutputs:
    """
    TRANSFORM AND RESAMPLE A VOLUME FILE.
    
    Resample a volume file with an arbitrary list of transformations. You may
    specify -affine, -warp, and -affine-series multiple times each, and they
    will be used in the order specified. For instance, for rigid motion
    correction followed by nonlinear atlas registration, specify -affine-series
    first, then -warp. The recommended methods are CUBIC (cubic spline) for most
    data, and ENCLOSING_VOXEL for label data. The parameter <method> must be one
    of:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR.
    
    Args:
        volume_in: volume to resample.
        volume_space: a volume file in the volume space you want for the output.
        method: the resampling method.
        volume_out: the output volume.
        affine: add an affine transform.
        affine_series: add an independent affine per-frame.
        warp: add a nonlinear warpfield transform.
        value: use a specified value for locations outside the FoV of the input\
            image or warpfield(s)\
            \
            the value to use (default 0).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeResampleOutputs`).
    """
    params = volume_resample_params(
        volume_in=volume_in,
        volume_space=volume_space,
        method=method,
        volume_out=volume_out,
        affine=affine,
        affine_series=affine_series,
        warp=warp,
        value=value,
    )
    return volume_resample_execute(params, runner)


__all__ = [
    "VOLUME_RESAMPLE_METADATA",
    "VolumeResampleAffineParamsDict",
    "VolumeResampleAffineParamsDictTagged",
    "VolumeResampleAffineSeriesParamsDict",
    "VolumeResampleAffineSeriesParamsDictTagged",
    "VolumeResampleFlirtParamsDict",
    "VolumeResampleFlirtParamsDictTagged",
    "VolumeResampleFlirtParamsDictTagged_",
    "VolumeResampleFlirtParamsDict_",
    "VolumeResampleOutputs",
    "VolumeResampleParamsDict",
    "VolumeResampleParamsDictTagged",
    "VolumeResampleWarpParamsDict",
    "VolumeResampleWarpParamsDictTagged",
    "volume_resample",
    "volume_resample_affine",
    "volume_resample_affine_series",
    "volume_resample_execute",
    "volume_resample_flirt",
    "volume_resample_flirt_",
    "volume_resample_params",
    "volume_resample_warp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_WEIGHTED_STATS_METADATA = Metadata(
    id="59d1210ae1d155885888b93e8db71154f04e52bd.workbench",
    name="metric-weighted-stats",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricWeightedStatsRoiParamsDictNoTag = typing.TypedDict('_MetricWeightedStatsRoiParamsDictNoTag', {
    "roi-metric": InputPathType,
    "match-maps": bool,
})
MetricWeightedStatsRoiParamsDictTagged = typing.TypedDict('MetricWeightedStatsRoiParamsDictTagged', {
    "@type": typing.Literal["roi"],
    "roi-metric": InputPathType,
    "match-maps": bool,
})
MetricWeightedStatsRoiParamsDict = _MetricWeightedStatsRoiParamsDictNoTag | MetricWeightedStatsRoiParamsDictTagged


_MetricWeightedStatsParamsDictNoTag = typing.TypedDict('_MetricWeightedStatsParamsDictNoTag', {
    "metric-in": InputPathType,
    "roi": typing.NotRequired[MetricWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "column": typing.NotRequired[str | None],
    "weight-metric": typing.NotRequired[InputPathType | None],
    "area-surface": typing.NotRequired[InputPathType | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
MetricWeightedStatsParamsDictTagged = typing.TypedDict('MetricWeightedStatsParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-weighted-stats"],
    "metric-in": InputPathType,
    "roi": typing.NotRequired[MetricWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "column": typing.NotRequired[str | None],
    "weight-metric": typing.NotRequired[InputPathType | None],
    "area-surface": typing.NotRequired[InputPathType | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
MetricWeightedStatsParamsDict = _MetricWeightedStatsParamsDictNoTag | MetricWeightedStatsParamsDictTagged


def metric_weighted_stats_roi(
    roi_metric: InputPathType,
    match_maps: bool = False,
) -> MetricWeightedStatsRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_metric: the roi, as a metric file.
        match_maps: each column of input uses the corresponding column from the\
            roi file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-metric": roi_metric,
        "match-maps": match_maps,
    }
    return params


def metric_weighted_stats_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricWeightedStatsRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-metric", None) is None:
        raise StyxValidationError("`roi-metric` must not be None")
    if not isinstance(params["roi-metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType`')
    if params.get("match-maps", False) is None:
        raise StyxValidationError("`match-maps` must not be None")
    if not isinstance(params["match-maps"], bool):
        raise StyxValidationError(f'`match-maps` has the wrong type: Received `{type(params.get("match-maps", False))}` expected `bool`')


def metric_weighted_stats_roi_cargs(
    params: MetricWeightedStatsRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        execution.input_file(params.get("roi-metric", None))
    ])
    if params.get("match-maps", False):
        cargs.append("-match-maps")
    return cargs


class MetricWeightedStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricWeightedStatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metric_weighted_stats_params(
    metric_in: InputPathType,
    roi: MetricWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    column: str | None = None,
    weight_metric: InputPathType | None = None,
    area_surface: InputPathType | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
) -> MetricWeightedStatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric_in: the input metric.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        column: only display output for one column\
            \
            the column number or name.
        weight_metric: use weights from a metric file\
            \
            metric file containing the weights.
        area_surface: use vertex areas as weights\
            \
            the surface to use for vertex areas.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-weighted-stats",
        "metric-in": metric_in,
        "show-map-name": show_map_name,
        "sum": sum_,
        "mean": mean,
    }
    if roi is not None:
        params["roi"] = roi
    if percent is not None:
        params["percent"] = percent
    if sample is not None:
        params["sample"] = sample
    if column is not None:
        params["column"] = column
    if weight_metric is not None:
        params["weight-metric"] = weight_metric
    if area_surface is not None:
        params["area-surface"] = area_surface
    return params


def metric_weighted_stats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricWeightedStatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("roi", None) is not None:
        metric_weighted_stats_roi_validate(params["roi"])
    if params.get("percent", None) is not None:
        if not isinstance(params["percent"], (float, int)):
            raise StyxValidationError(f'`percent` has the wrong type: Received `{type(params.get("percent", None))}` expected `float | None`')
    if params.get("sample", None) is not None:
        if not isinstance(params["sample"], bool):
            raise StyxValidationError(f'`sample` has the wrong type: Received `{type(params.get("sample", None))}` expected `bool | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("weight-metric", None) is not None:
        if not isinstance(params["weight-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weight-metric` has the wrong type: Received `{type(params.get("weight-metric", None))}` expected `InputPathType | None`')
    if params.get("area-surface", None) is not None:
        if not isinstance(params["area-surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-surface` has the wrong type: Received `{type(params.get("area-surface", None))}` expected `InputPathType | None`')
    if params.get("show-map-name", False) is None:
        raise StyxValidationError("`show-map-name` must not be None")
    if not isinstance(params["show-map-name"], bool):
        raise StyxValidationError(f'`show-map-name` has the wrong type: Received `{type(params.get("show-map-name", False))}` expected `bool`')
    if params.get("sum", False) is None:
        raise StyxValidationError("`sum` must not be None")
    if not isinstance(params["sum"], bool):
        raise StyxValidationError(f'`sum` has the wrong type: Received `{type(params.get("sum", False))}` expected `bool`')
    if params.get("mean", False) is None:
        raise StyxValidationError("`mean` must not be None")
    if not isinstance(params["mean"], bool):
        raise StyxValidationError(f'`mean` has the wrong type: Received `{type(params.get("mean", False))}` expected `bool`')


def metric_weighted_stats_cargs(
    params: MetricWeightedStatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-weighted-stats"
    ])
    cargs.append(execution.input_file(params.get("metric-in", None)))
    if params.get("roi", None) is not None:
        cargs.extend(metric_weighted_stats_roi_cargs(params.get("roi", None), execution))
    if params.get("percent", None) is not None:
        cargs.extend([
            "-percentile",
            str(params.get("percent", None))
        ])
    if params.get("sample", None) is not None:
        cargs.extend([
            "-stdev",
            "-sample"
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("weight-metric", None) is not None:
        cargs.extend([
            "-weight-metric",
            execution.input_file(params.get("weight-metric", None))
        ])
    if params.get("area-surface", None) is not None:
        cargs.extend([
            "-area-surface",
            execution.input_file(params.get("area-surface", None))
        ])
    if params.get("show-map-name", False):
        cargs.append("-show-map-name")
    if params.get("sum", False):
        cargs.append("-sum")
    if params.get("mean", False):
        cargs.append("-mean")
    return cargs


def metric_weighted_stats_outputs(
    params: MetricWeightedStatsParamsDict,
    execution: Execution,
) -> MetricWeightedStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricWeightedStatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def metric_weighted_stats_execute(
    params: MetricWeightedStatsParamsDict,
    runner: Runner | None = None,
) -> MetricWeightedStatsOutputs:
    """
    WEIGHTED SPATIAL STATISTICS ON A METRIC FILE.
    
    For each column of the input, a line of text is printed, resulting from the
    specified operation. You must specify exactly one of -area-surface or
    -weight-metric. Use -column to only give output for a single column. If the
    -roi option is used without -match-maps, then each line will contain as many
    numbers as there are maps in the ROI file, separated by tab characters.
    Exactly one of -mean, -stdev, -percentile or -sum must be specified.
    
    Using -sum with -area-surface (or -weight-metric with a metric containing
    similar data) is equivalent to integrating with respect to surface area. For
    example, if you want to find the surface area within an roi, do this:
    
    $ wb_command -metric-weighted-stats roi.func.gii -sum -area-surface
    midthickness.surf.gii.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricWeightedStatsOutputs`).
    """
    metric_weighted_stats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_WEIGHTED_STATS_METADATA)
    params = execution.params(params)
    cargs = metric_weighted_stats_cargs(params, execution)
    ret = metric_weighted_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_weighted_stats(
    metric_in: InputPathType,
    roi: MetricWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    column: str | None = None,
    weight_metric: InputPathType | None = None,
    area_surface: InputPathType | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
    runner: Runner | None = None,
) -> MetricWeightedStatsOutputs:
    """
    WEIGHTED SPATIAL STATISTICS ON A METRIC FILE.
    
    For each column of the input, a line of text is printed, resulting from the
    specified operation. You must specify exactly one of -area-surface or
    -weight-metric. Use -column to only give output for a single column. If the
    -roi option is used without -match-maps, then each line will contain as many
    numbers as there are maps in the ROI file, separated by tab characters.
    Exactly one of -mean, -stdev, -percentile or -sum must be specified.
    
    Using -sum with -area-surface (or -weight-metric with a metric containing
    similar data) is equivalent to integrating with respect to surface area. For
    example, if you want to find the surface area within an roi, do this:
    
    $ wb_command -metric-weighted-stats roi.func.gii -sum -area-surface
    midthickness.surf.gii.
    
    Args:
        metric_in: the input metric.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        column: only display output for one column\
            \
            the column number or name.
        weight_metric: use weights from a metric file\
            \
            metric file containing the weights.
        area_surface: use vertex areas as weights\
            \
            the surface to use for vertex areas.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricWeightedStatsOutputs`).
    """
    params = metric_weighted_stats_params(
        metric_in=metric_in,
        roi=roi,
        percent=percent,
        sample=sample,
        column=column,
        weight_metric=weight_metric,
        area_surface=area_surface,
        show_map_name=show_map_name,
        sum_=sum_,
        mean=mean,
    )
    return metric_weighted_stats_execute(params, runner)


__all__ = [
    "METRIC_WEIGHTED_STATS_METADATA",
    "MetricWeightedStatsOutputs",
    "MetricWeightedStatsParamsDict",
    "MetricWeightedStatsParamsDictTagged",
    "MetricWeightedStatsRoiParamsDict",
    "MetricWeightedStatsRoiParamsDictTagged",
    "metric_weighted_stats",
    "metric_weighted_stats_execute",
    "metric_weighted_stats_params",
    "metric_weighted_stats_roi",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SCENE_FILE_MERGE_METADATA = Metadata(
    id="72ce88135b0199024083ada6e5b4934b29d0f4bf.workbench",
    name="scene-file-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SceneFileMergeUpToParamsDictNoTag = typing.TypedDict('_SceneFileMergeUpToParamsDictNoTag', {
    "last-column": str,
    "reverse": bool,
})
SceneFileMergeUpToParamsDictTagged = typing.TypedDict('SceneFileMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-column": str,
    "reverse": bool,
})
SceneFileMergeUpToParamsDict = _SceneFileMergeUpToParamsDictNoTag | SceneFileMergeUpToParamsDictTagged


_SceneFileMergeSceneParamsDictNoTag = typing.TypedDict('_SceneFileMergeSceneParamsDictNoTag', {
    "scene": str,
    "up-to": typing.NotRequired[SceneFileMergeUpToParamsDict | None],
})
SceneFileMergeSceneParamsDictTagged = typing.TypedDict('SceneFileMergeSceneParamsDictTagged', {
    "@type": typing.Literal["scene"],
    "scene": str,
    "up-to": typing.NotRequired[SceneFileMergeUpToParamsDict | None],
})
SceneFileMergeSceneParamsDict = _SceneFileMergeSceneParamsDictNoTag | SceneFileMergeSceneParamsDictTagged


_SceneFileMergeSceneFileParamsDictNoTag = typing.TypedDict('_SceneFileMergeSceneFileParamsDictNoTag', {
    "scene-file": str,
    "scene": typing.NotRequired[list[SceneFileMergeSceneParamsDict] | None],
})
SceneFileMergeSceneFileParamsDictTagged = typing.TypedDict('SceneFileMergeSceneFileParamsDictTagged', {
    "@type": typing.Literal["scene-file"],
    "scene-file": str,
    "scene": typing.NotRequired[list[SceneFileMergeSceneParamsDict] | None],
})
SceneFileMergeSceneFileParamsDict = _SceneFileMergeSceneFileParamsDictNoTag | SceneFileMergeSceneFileParamsDictTagged


_SceneFileMergeParamsDictNoTag = typing.TypedDict('_SceneFileMergeParamsDictNoTag', {
    "scene-file-out": str,
    "scene-file": typing.NotRequired[list[SceneFileMergeSceneFileParamsDict] | None],
})
SceneFileMergeParamsDictTagged = typing.TypedDict('SceneFileMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/scene-file-merge"],
    "scene-file-out": str,
    "scene-file": typing.NotRequired[list[SceneFileMergeSceneFileParamsDict] | None],
})
SceneFileMergeParamsDict = _SceneFileMergeParamsDictNoTag | SceneFileMergeParamsDictTagged


def scene_file_merge_up_to(
    last_column: str,
    reverse: bool = False,
) -> SceneFileMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_column: the number or name of the last scene to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-column": last_column,
        "reverse": reverse,
    }
    return params


def scene_file_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneFileMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-column", None) is None:
        raise StyxValidationError("`last-column` must not be None")
    if not isinstance(params["last-column"], str):
        raise StyxValidationError(f'`last-column` has the wrong type: Received `{type(params.get("last-column", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def scene_file_merge_up_to_cargs(
    params: SceneFileMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-column", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def scene_file_merge_scene(
    scene: str,
    up_to: SceneFileMergeUpToParamsDict | None = None,
) -> SceneFileMergeSceneParamsDictTagged:
    """
    Build parameters.
    
    Args:
        scene: the scene number or name.
        up_to: use an inclusive range of scenes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "scene",
        "scene": scene,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def scene_file_merge_scene_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneFileMergeSceneParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("scene", None) is None:
        raise StyxValidationError("`scene` must not be None")
    if not isinstance(params["scene"], str):
        raise StyxValidationError(f'`scene` has the wrong type: Received `{type(params.get("scene", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        scene_file_merge_up_to_validate(params["up-to"])


def scene_file_merge_scene_cargs(
    params: SceneFileMergeSceneParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-scene",
        params.get("scene", None),
        *(scene_file_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def scene_file_merge_scene_file(
    scene_file: str,
    scene: list[SceneFileMergeSceneParamsDict] | None = None,
) -> SceneFileMergeSceneFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        scene_file: the input scene file.
        scene: specify a scene to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "scene-file",
        "scene-file": scene_file,
    }
    if scene is not None:
        params["scene"] = scene
    return params


def scene_file_merge_scene_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneFileMergeSceneFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("scene-file", None) is None:
        raise StyxValidationError("`scene-file` must not be None")
    if not isinstance(params["scene-file"], str):
        raise StyxValidationError(f'`scene-file` has the wrong type: Received `{type(params.get("scene-file", None))}` expected `str`')
    if params.get("scene", None) is not None:
        if not isinstance(params["scene"], list):
            raise StyxValidationError(f'`scene` has the wrong type: Received `{type(params.get("scene", None))}` expected `list[SceneFileMergeSceneParamsDict] | None`')
        for e in params["scene"]:
            scene_file_merge_scene_validate(e)


def scene_file_merge_scene_file_cargs(
    params: SceneFileMergeSceneFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-scene-file",
        params.get("scene-file", None),
        *([a for c in [scene_file_merge_scene_cargs(s, execution) for s in params.get("scene", None)] for a in c] if (params.get("scene", None) is not None) else [])
    ])
    return cargs


class SceneFileMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SceneFileMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_file_merge_params(
    scene_file_out: str,
    scene_file: list[SceneFileMergeSceneFileParamsDict] | None = None,
) -> SceneFileMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        scene_file_out: output - the output scene file.
        scene_file: specify a scene file to use scenes from.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/scene-file-merge",
        "scene-file-out": scene_file_out,
    }
    if scene_file is not None:
        params["scene-file"] = scene_file
    return params


def scene_file_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneFileMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("scene-file-out", None) is None:
        raise StyxValidationError("`scene-file-out` must not be None")
    if not isinstance(params["scene-file-out"], str):
        raise StyxValidationError(f'`scene-file-out` has the wrong type: Received `{type(params.get("scene-file-out", None))}` expected `str`')
    if params.get("scene-file", None) is not None:
        if not isinstance(params["scene-file"], list):
            raise StyxValidationError(f'`scene-file` has the wrong type: Received `{type(params.get("scene-file", None))}` expected `list[SceneFileMergeSceneFileParamsDict] | None`')
        for e in params["scene-file"]:
            scene_file_merge_scene_file_validate(e)


def scene_file_merge_cargs(
    params: SceneFileMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-scene-file-merge"
    ])
    cargs.append(params.get("scene-file-out", None))
    if params.get("scene-file", None) is not None:
        cargs.extend([a for c in [scene_file_merge_scene_file_cargs(s, execution) for s in params.get("scene-file", None)] for a in c])
    return cargs


def scene_file_merge_outputs(
    params: SceneFileMergeParamsDict,
    execution: Execution,
) -> SceneFileMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SceneFileMergeOutputs(
        root=execution.output_file("."),
    )
    return ret


def scene_file_merge_execute(
    params: SceneFileMergeParamsDict,
    runner: Runner | None = None,
) -> SceneFileMergeOutputs:
    """
    REARRANGE SCENES INTO A NEW FILE.
    
    Takes one or more scene files and constructs a new scene file by
    concatenating specified scenes from them.
    
    Example: wb_command -scene-file-merge out.scene -scene-file first.scene
    -scene 1 -scene-file second.scene
    
    This example would take the first scene from first.scene, followed by all
    scenes from second.scene, and write these scenes to out.scene.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileMergeOutputs`).
    """
    scene_file_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_FILE_MERGE_METADATA)
    params = execution.params(params)
    cargs = scene_file_merge_cargs(params, execution)
    ret = scene_file_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def scene_file_merge(
    scene_file_out: str,
    scene_file: list[SceneFileMergeSceneFileParamsDict] | None = None,
    runner: Runner | None = None,
) -> SceneFileMergeOutputs:
    """
    REARRANGE SCENES INTO A NEW FILE.
    
    Takes one or more scene files and constructs a new scene file by
    concatenating specified scenes from them.
    
    Example: wb_command -scene-file-merge out.scene -scene-file first.scene
    -scene 1 -scene-file second.scene
    
    This example would take the first scene from first.scene, followed by all
    scenes from second.scene, and write these scenes to out.scene.
    
    Args:
        scene_file_out: output - the output scene file.
        scene_file: specify a scene file to use scenes from.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneFileMergeOutputs`).
    """
    params = scene_file_merge_params(
        scene_file_out=scene_file_out,
        scene_file=scene_file,
    )
    return scene_file_merge_execute(params, runner)


__all__ = [
    "SCENE_FILE_MERGE_METADATA",
    "SceneFileMergeOutputs",
    "SceneFileMergeParamsDict",
    "SceneFileMergeParamsDictTagged",
    "SceneFileMergeSceneFileParamsDict",
    "SceneFileMergeSceneFileParamsDictTagged",
    "SceneFileMergeSceneParamsDict",
    "SceneFileMergeSceneParamsDictTagged",
    "SceneFileMergeUpToParamsDict",
    "SceneFileMergeUpToParamsDictTagged",
    "scene_file_merge",
    "scene_file_merge_execute",
    "scene_file_merge_params",
    "scene_file_merge_scene",
    "scene_file_merge_scene_file",
    "scene_file_merge_up_to",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_PROBABILITY_METADATA = Metadata(
    id="3036af31e9086ad81e72bfa02cf68fef734a77c9.workbench",
    name="label-probability",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelProbabilityParamsDictNoTag = typing.TypedDict('_LabelProbabilityParamsDictNoTag', {
    "label-maps": InputPathType,
    "probability-metric-out": str,
    "exclude-unlabeled": bool,
})
LabelProbabilityParamsDictTagged = typing.TypedDict('LabelProbabilityParamsDictTagged', {
    "@type": typing.Literal["workbench/label-probability"],
    "label-maps": InputPathType,
    "probability-metric-out": str,
    "exclude-unlabeled": bool,
})
LabelProbabilityParamsDict = _LabelProbabilityParamsDictNoTag | LabelProbabilityParamsDictTagged


class LabelProbabilityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelProbabilityParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    probability_metric_out: OutputPathType
    """the relative frequencies of each label at each vertex"""


def label_probability_params(
    label_maps: InputPathType,
    probability_metric_out: str,
    exclude_unlabeled: bool = False,
) -> LabelProbabilityParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_maps: label file containing individual label maps from many\
            subjects.
        probability_metric_out: the relative frequencies of each label at each\
            vertex.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-probability",
        "label-maps": label_maps,
        "probability-metric-out": probability_metric_out,
        "exclude-unlabeled": exclude_unlabeled,
    }
    return params


def label_probability_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelProbabilityParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-maps", None) is None:
        raise StyxValidationError("`label-maps` must not be None")
    if not isinstance(params["label-maps"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-maps` has the wrong type: Received `{type(params.get("label-maps", None))}` expected `InputPathType`')
    if params.get("probability-metric-out", None) is None:
        raise StyxValidationError("`probability-metric-out` must not be None")
    if not isinstance(params["probability-metric-out"], str):
        raise StyxValidationError(f'`probability-metric-out` has the wrong type: Received `{type(params.get("probability-metric-out", None))}` expected `str`')
    if params.get("exclude-unlabeled", False) is None:
        raise StyxValidationError("`exclude-unlabeled` must not be None")
    if not isinstance(params["exclude-unlabeled"], bool):
        raise StyxValidationError(f'`exclude-unlabeled` has the wrong type: Received `{type(params.get("exclude-unlabeled", False))}` expected `bool`')


def label_probability_cargs(
    params: LabelProbabilityParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-probability"
    ])
    cargs.append(execution.input_file(params.get("label-maps", None)))
    cargs.append(params.get("probability-metric-out", None))
    if params.get("exclude-unlabeled", False):
        cargs.append("-exclude-unlabeled")
    return cargs


def label_probability_outputs(
    params: LabelProbabilityParamsDict,
    execution: Execution,
) -> LabelProbabilityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelProbabilityOutputs(
        root=execution.output_file("."),
        probability_metric_out=execution.output_file(params.get("probability-metric-out", None)),
    )
    return ret


def label_probability_execute(
    params: LabelProbabilityParamsDict,
    runner: Runner | None = None,
) -> LabelProbabilityOutputs:
    """
    FIND FREQUENCY OF SURFACE LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that vertex,
    divided by the number of input maps.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelProbabilityOutputs`).
    """
    label_probability_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_PROBABILITY_METADATA)
    params = execution.params(params)
    cargs = label_probability_cargs(params, execution)
    ret = label_probability_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_probability(
    label_maps: InputPathType,
    probability_metric_out: str,
    exclude_unlabeled: bool = False,
    runner: Runner | None = None,
) -> LabelProbabilityOutputs:
    """
    FIND FREQUENCY OF SURFACE LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that vertex,
    divided by the number of input maps.
    
    Args:
        label_maps: label file containing individual label maps from many\
            subjects.
        probability_metric_out: the relative frequencies of each label at each\
            vertex.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelProbabilityOutputs`).
    """
    params = label_probability_params(
        label_maps=label_maps,
        probability_metric_out=probability_metric_out,
        exclude_unlabeled=exclude_unlabeled,
    )
    return label_probability_execute(params, runner)


__all__ = [
    "LABEL_PROBABILITY_METADATA",
    "LabelProbabilityOutputs",
    "LabelProbabilityParamsDict",
    "LabelProbabilityParamsDictTagged",
    "label_probability",
    "label_probability_execute",
    "label_probability_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_GRADIENT_METADATA = Metadata(
    id="5a5d7ede042330cd65eec319159b5ecf75f94aed.workbench",
    name="volume-gradient",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeGradientPresmoothParamsDictNoTag = typing.TypedDict('_VolumeGradientPresmoothParamsDictNoTag', {
    "kernel": float,
    "fwhm": bool,
})
VolumeGradientPresmoothParamsDictTagged = typing.TypedDict('VolumeGradientPresmoothParamsDictTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "fwhm": bool,
})
VolumeGradientPresmoothParamsDict = _VolumeGradientPresmoothParamsDictNoTag | VolumeGradientPresmoothParamsDictTagged


_VolumeGradientVectorsParamsDictNoTag = typing.TypedDict('_VolumeGradientVectorsParamsDictNoTag', {
    "vector-volume-out": str,
})
VolumeGradientVectorsParamsDictTagged = typing.TypedDict('VolumeGradientVectorsParamsDictTagged', {
    "@type": typing.Literal["vectors"],
    "vector-volume-out": str,
})
VolumeGradientVectorsParamsDict = _VolumeGradientVectorsParamsDictNoTag | VolumeGradientVectorsParamsDictTagged


_VolumeGradientParamsDictNoTag = typing.TypedDict('_VolumeGradientParamsDictNoTag', {
    "volume-in": InputPathType,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeGradientPresmoothParamsDict | None],
    "vectors": typing.NotRequired[VolumeGradientVectorsParamsDict | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeGradientParamsDictTagged = typing.TypedDict('VolumeGradientParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-gradient"],
    "volume-in": InputPathType,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeGradientPresmoothParamsDict | None],
    "vectors": typing.NotRequired[VolumeGradientVectorsParamsDict | None],
    "subvol": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeGradientParamsDict = _VolumeGradientParamsDictNoTag | VolumeGradientParamsDictTagged


def volume_gradient_presmooth(
    kernel: float,
    fwhm: bool = False,
) -> VolumeGradientPresmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "fwhm": fwhm,
    }
    return params


def volume_gradient_presmooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeGradientPresmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_gradient_presmooth_cargs(
    params: VolumeGradientPresmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-presmooth",
        str(params.get("kernel", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


class VolumeGradientVectorsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeGradientVectorsParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    vector_volume_out: OutputPathType
    """the vectors as a volume file"""


def volume_gradient_vectors(
    vector_volume_out: str,
) -> VolumeGradientVectorsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        vector_volume_out: the vectors as a volume file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "vectors",
        "vector-volume-out": vector_volume_out,
    }
    return params


def volume_gradient_vectors_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeGradientVectorsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("vector-volume-out", None) is None:
        raise StyxValidationError("`vector-volume-out` must not be None")
    if not isinstance(params["vector-volume-out"], str):
        raise StyxValidationError(f'`vector-volume-out` has the wrong type: Received `{type(params.get("vector-volume-out", None))}` expected `str`')


def volume_gradient_vectors_cargs(
    params: VolumeGradientVectorsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-vectors",
        params.get("vector-volume-out", None)
    ])
    return cargs


def volume_gradient_vectors_outputs(
    params: VolumeGradientVectorsParamsDict,
    execution: Execution,
) -> VolumeGradientVectorsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeGradientVectorsOutputs(
        root=execution.output_file("."),
        vector_volume_out=execution.output_file(params.get("vector-volume-out", None)),
    )
    return ret


class VolumeGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeGradientParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output gradient magnitude volume"""
    vectors: VolumeGradientVectorsOutputs | None
    """Outputs from `volume_gradient_vectors_outputs`."""


def volume_gradient_params(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeGradientPresmoothParamsDict | None = None,
    vectors: VolumeGradientVectorsParamsDict | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
) -> VolumeGradientParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        volume_out: the output gradient magnitude volume.
        presmooth: smooth the volume before computing the gradient.
        vectors: output vectors.
        subvol: select a single subvolume to take the gradient of\
            \
            the subvolume number or name.
        roi_volume: select a region of interest to take the gradient of\
            \
            the region to take the gradient within.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-gradient",
        "volume-in": volume_in,
        "volume-out": volume_out,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if vectors is not None:
        params["vectors"] = vectors
    if subvol is not None:
        params["subvol"] = subvol
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    return params


def volume_gradient_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeGradientParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("presmooth", None) is not None:
        volume_gradient_presmooth_validate(params["presmooth"])
    if params.get("vectors", None) is not None:
        volume_gradient_vectors_validate(params["vectors"])
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')


def volume_gradient_cargs(
    params: VolumeGradientParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-gradient"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("presmooth", None) is not None or params.get("vectors", None) is not None:
        cargs.extend([
            *(volume_gradient_presmooth_cargs(params.get("presmooth", None), execution) if (params.get("presmooth", None) is not None) else []),
            *(volume_gradient_vectors_cargs(params.get("vectors", None), execution) if (params.get("vectors", None) is not None) else [])
        ])
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    return cargs


def volume_gradient_outputs(
    params: VolumeGradientParamsDict,
    execution: Execution,
) -> VolumeGradientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeGradientOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
        vectors=volume_gradient_vectors_outputs(params.get("vectors"), execution) if params.get("vectors") else None,
    )
    return ret


def volume_gradient_execute(
    params: VolumeGradientParamsDict,
    runner: Runner | None = None,
) -> VolumeGradientOutputs:
    """
    GRADIENT OF A VOLUME FILE.
    
    Computes the gradient of the volume by doing linear regressions for each
    voxel, considering only its face neighbors unless too few face neighbors
    exist. The gradient vector is constructed from the partial derivatives of
    the resulting linear function, and the magnitude of this vector is the
    output. If specified, the volume vector output is arranged with the x, y,
    and z components from a subvolume as consecutive subvolumes.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeGradientOutputs`).
    """
    volume_gradient_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_GRADIENT_METADATA)
    params = execution.params(params)
    cargs = volume_gradient_cargs(params, execution)
    ret = volume_gradient_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_gradient(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeGradientPresmoothParamsDict | None = None,
    vectors: VolumeGradientVectorsParamsDict | None = None,
    subvol: str | None = None,
    roi_volume: InputPathType | None = None,
    runner: Runner | None = None,
) -> VolumeGradientOutputs:
    """
    GRADIENT OF A VOLUME FILE.
    
    Computes the gradient of the volume by doing linear regressions for each
    voxel, considering only its face neighbors unless too few face neighbors
    exist. The gradient vector is constructed from the partial derivatives of
    the resulting linear function, and the magnitude of this vector is the
    output. If specified, the volume vector output is arranged with the x, y,
    and z components from a subvolume as consecutive subvolumes.
    
    Args:
        volume_in: the input volume.
        volume_out: the output gradient magnitude volume.
        presmooth: smooth the volume before computing the gradient.
        vectors: output vectors.
        subvol: select a single subvolume to take the gradient of\
            \
            the subvolume number or name.
        roi_volume: select a region of interest to take the gradient of\
            \
            the region to take the gradient within.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeGradientOutputs`).
    """
    params = volume_gradient_params(
        volume_in=volume_in,
        volume_out=volume_out,
        presmooth=presmooth,
        vectors=vectors,
        subvol=subvol,
        roi_volume=roi_volume,
    )
    return volume_gradient_execute(params, runner)


__all__ = [
    "VOLUME_GRADIENT_METADATA",
    "VolumeGradientOutputs",
    "VolumeGradientParamsDict",
    "VolumeGradientParamsDictTagged",
    "VolumeGradientPresmoothParamsDict",
    "VolumeGradientPresmoothParamsDictTagged",
    "VolumeGradientVectorsOutputs",
    "VolumeGradientVectorsParamsDict",
    "VolumeGradientVectorsParamsDictTagged",
    "volume_gradient",
    "volume_gradient_execute",
    "volume_gradient_params",
    "volume_gradient_presmooth",
    "volume_gradient_vectors",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_TO_VOLUME_MAPPING_METADATA = Metadata(
    id="59e64c901ea699276bc6abbb818ee42c40042ca0.workbench",
    name="metric-to-volume-mapping",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricToVolumeMappingRibbonConstrainedParamsDictNoTag = typing.TypedDict('_MetricToVolumeMappingRibbonConstrainedParamsDictNoTag', {
    "inner-surf": InputPathType,
    "outer-surf": InputPathType,
    "subdiv-num": typing.NotRequired[int | None],
    "thick-columns": bool,
    "greedy": bool,
})
MetricToVolumeMappingRibbonConstrainedParamsDictTagged = typing.TypedDict('MetricToVolumeMappingRibbonConstrainedParamsDictTagged', {
    "@type": typing.Literal["ribbon-constrained"],
    "inner-surf": InputPathType,
    "outer-surf": InputPathType,
    "subdiv-num": typing.NotRequired[int | None],
    "thick-columns": bool,
    "greedy": bool,
})
MetricToVolumeMappingRibbonConstrainedParamsDict = _MetricToVolumeMappingRibbonConstrainedParamsDictNoTag | MetricToVolumeMappingRibbonConstrainedParamsDictTagged


_MetricToVolumeMappingParamsDictNoTag = typing.TypedDict('_MetricToVolumeMappingParamsDictNoTag', {
    "metric": InputPathType,
    "surface": InputPathType,
    "volume-space": InputPathType,
    "volume-out": str,
    "ribbon-constrained": typing.NotRequired[MetricToVolumeMappingRibbonConstrainedParamsDict | None],
    "distance": typing.NotRequired[float | None],
})
MetricToVolumeMappingParamsDictTagged = typing.TypedDict('MetricToVolumeMappingParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-to-volume-mapping"],
    "metric": InputPathType,
    "surface": InputPathType,
    "volume-space": InputPathType,
    "volume-out": str,
    "ribbon-constrained": typing.NotRequired[MetricToVolumeMappingRibbonConstrainedParamsDict | None],
    "distance": typing.NotRequired[float | None],
})
MetricToVolumeMappingParamsDict = _MetricToVolumeMappingParamsDictNoTag | MetricToVolumeMappingParamsDictTagged


def metric_to_volume_mapping_ribbon_constrained(
    inner_surf: InputPathType,
    outer_surf: InputPathType,
    subdiv_num: int | None = None,
    thick_columns: bool = False,
    greedy: bool = False,
) -> MetricToVolumeMappingRibbonConstrainedParamsDictTagged:
    """
    Build parameters.
    
    Args:
        inner_surf: the inner surface of the ribbon.
        outer_surf: the outer surface of the ribbon.
        subdiv_num: voxel divisions while estimating voxel weights\
            \
            number of subdivisions, default 3.
        thick_columns: use overlapping columns (legacy method).
        greedy: instead of antialiasing partial-volumed voxels, put full metric\
            values (legacy behavior).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "ribbon-constrained",
        "inner-surf": inner_surf,
        "outer-surf": outer_surf,
        "thick-columns": thick_columns,
        "greedy": greedy,
    }
    if subdiv_num is not None:
        params["subdiv-num"] = subdiv_num
    return params


def metric_to_volume_mapping_ribbon_constrained_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricToVolumeMappingRibbonConstrainedParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("inner-surf", None) is None:
        raise StyxValidationError("`inner-surf` must not be None")
    if not isinstance(params["inner-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`inner-surf` has the wrong type: Received `{type(params.get("inner-surf", None))}` expected `InputPathType`')
    if params.get("outer-surf", None) is None:
        raise StyxValidationError("`outer-surf` must not be None")
    if not isinstance(params["outer-surf"], (pathlib.Path, str)):
        raise StyxValidationError(f'`outer-surf` has the wrong type: Received `{type(params.get("outer-surf", None))}` expected `InputPathType`')
    if params.get("subdiv-num", None) is not None:
        if not isinstance(params["subdiv-num"], int):
            raise StyxValidationError(f'`subdiv-num` has the wrong type: Received `{type(params.get("subdiv-num", None))}` expected `int | None`')
    if params.get("thick-columns", False) is None:
        raise StyxValidationError("`thick-columns` must not be None")
    if not isinstance(params["thick-columns"], bool):
        raise StyxValidationError(f'`thick-columns` has the wrong type: Received `{type(params.get("thick-columns", False))}` expected `bool`')
    if params.get("greedy", False) is None:
        raise StyxValidationError("`greedy` must not be None")
    if not isinstance(params["greedy"], bool):
        raise StyxValidationError(f'`greedy` has the wrong type: Received `{type(params.get("greedy", False))}` expected `bool`')


def metric_to_volume_mapping_ribbon_constrained_cargs(
    params: MetricToVolumeMappingRibbonConstrainedParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-ribbon-constrained",
        execution.input_file(params.get("inner-surf", None)),
        execution.input_file(params.get("outer-surf", None))
    ])
    if params.get("subdiv-num", None) is not None:
        cargs.extend([
            "-voxel-subdiv",
            str(params.get("subdiv-num", None))
        ])
    if params.get("thick-columns", False):
        cargs.append("-thick-columns")
    if params.get("greedy", False):
        cargs.append("-greedy")
    return cargs


class MetricToVolumeMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricToVolumeMappingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def metric_to_volume_mapping_params(
    metric: InputPathType,
    surface: InputPathType,
    volume_space: InputPathType,
    volume_out: str,
    ribbon_constrained: MetricToVolumeMappingRibbonConstrainedParamsDict | None = None,
    distance: float | None = None,
) -> MetricToVolumeMappingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the input metric file.
        surface: the surface to use coordinates from.
        volume_space: a volume file in the desired output volume space.
        volume_out: the output volume file.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        distance: use the value from the vertex closest to the voxel center\
            \
            how far from the surface to map values to voxels, in mm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-to-volume-mapping",
        "metric": metric,
        "surface": surface,
        "volume-space": volume_space,
        "volume-out": volume_out,
    }
    if ribbon_constrained is not None:
        params["ribbon-constrained"] = ribbon_constrained
    if distance is not None:
        params["distance"] = distance
    return params


def metric_to_volume_mapping_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricToVolumeMappingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("volume-space", None) is None:
        raise StyxValidationError("`volume-space` must not be None")
    if not isinstance(params["volume-space"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-space` has the wrong type: Received `{type(params.get("volume-space", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("ribbon-constrained", None) is not None:
        metric_to_volume_mapping_ribbon_constrained_validate(params["ribbon-constrained"])
    if params.get("distance", None) is not None:
        if not isinstance(params["distance"], (float, int)):
            raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float | None`')


def metric_to_volume_mapping_cargs(
    params: MetricToVolumeMappingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-to-volume-mapping"
    ])
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("volume-space", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("ribbon-constrained", None) is not None:
        cargs.extend(metric_to_volume_mapping_ribbon_constrained_cargs(params.get("ribbon-constrained", None), execution))
    if params.get("distance", None) is not None:
        cargs.extend([
            "-nearest-vertex",
            str(params.get("distance", None))
        ])
    return cargs


def metric_to_volume_mapping_outputs(
    params: MetricToVolumeMappingParamsDict,
    execution: Execution,
) -> MetricToVolumeMappingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricToVolumeMappingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def metric_to_volume_mapping_execute(
    params: MetricToVolumeMappingParamsDict,
    runner: Runner | None = None,
) -> MetricToVolumeMappingOutputs:
    """
    MAP METRIC FILE TO VOLUME.
    
    Maps values from a metric file into a volume file. You must specify exactly
    one mapping method option. The -nearest-vertex method uses the value from
    the vertex closest to the voxel center (useful for integer values). The
    -ribbon-constrained method uses the same method as in
    -volume-to-surface-mapping, then uses the weights in reverse. Mapping to
    lower resolutions than the mesh may require a larger -voxel-subdiv value in
    order to have all of the surface data participate.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricToVolumeMappingOutputs`).
    """
    metric_to_volume_mapping_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_TO_VOLUME_MAPPING_METADATA)
    params = execution.params(params)
    cargs = metric_to_volume_mapping_cargs(params, execution)
    ret = metric_to_volume_mapping_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_to_volume_mapping(
    metric: InputPathType,
    surface: InputPathType,
    volume_space: InputPathType,
    volume_out: str,
    ribbon_constrained: MetricToVolumeMappingRibbonConstrainedParamsDict | None = None,
    distance: float | None = None,
    runner: Runner | None = None,
) -> MetricToVolumeMappingOutputs:
    """
    MAP METRIC FILE TO VOLUME.
    
    Maps values from a metric file into a volume file. You must specify exactly
    one mapping method option. The -nearest-vertex method uses the value from
    the vertex closest to the voxel center (useful for integer values). The
    -ribbon-constrained method uses the same method as in
    -volume-to-surface-mapping, then uses the weights in reverse. Mapping to
    lower resolutions than the mesh may require a larger -voxel-subdiv value in
    order to have all of the surface data participate.
    
    Args:
        metric: the input metric file.
        surface: the surface to use coordinates from.
        volume_space: a volume file in the desired output volume space.
        volume_out: the output volume file.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        distance: use the value from the vertex closest to the voxel center\
            \
            how far from the surface to map values to voxels, in mm.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricToVolumeMappingOutputs`).
    """
    params = metric_to_volume_mapping_params(
        metric=metric,
        surface=surface,
        volume_space=volume_space,
        volume_out=volume_out,
        ribbon_constrained=ribbon_constrained,
        distance=distance,
    )
    return metric_to_volume_mapping_execute(params, runner)


__all__ = [
    "METRIC_TO_VOLUME_MAPPING_METADATA",
    "MetricToVolumeMappingOutputs",
    "MetricToVolumeMappingParamsDict",
    "MetricToVolumeMappingParamsDictTagged",
    "MetricToVolumeMappingRibbonConstrainedParamsDict",
    "MetricToVolumeMappingRibbonConstrainedParamsDictTagged",
    "metric_to_volume_mapping",
    "metric_to_volume_mapping_execute",
    "metric_to_volume_mapping_params",
    "metric_to_volume_mapping_ribbon_constrained",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_PARCELLATE_METADATA = Metadata(
    id="7b2f49ca19798d787ca04a1e72ba218c146a3ab4.workbench",
    name="cifti-parcellate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiParcellateSpatialWeightsParamsDictNoTag = typing.TypedDict('_CiftiParcellateSpatialWeightsParamsDictNoTag', {
    "cerebellum-metric": typing.NotRequired[InputPathType | None],
    "right-metric": typing.NotRequired[InputPathType | None],
    "left-metric": typing.NotRequired[InputPathType | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
})
CiftiParcellateSpatialWeightsParamsDictTagged = typing.TypedDict('CiftiParcellateSpatialWeightsParamsDictTagged', {
    "@type": typing.Literal["spatial-weights"],
    "cerebellum-metric": typing.NotRequired[InputPathType | None],
    "right-metric": typing.NotRequired[InputPathType | None],
    "left-metric": typing.NotRequired[InputPathType | None],
    "cerebellum-surf": typing.NotRequired[InputPathType | None],
    "right-surf": typing.NotRequired[InputPathType | None],
    "left-surf": typing.NotRequired[InputPathType | None],
})
CiftiParcellateSpatialWeightsParamsDict = _CiftiParcellateSpatialWeightsParamsDictNoTag | CiftiParcellateSpatialWeightsParamsDictTagged


_CiftiParcellateExcludeOutliersParamsDictNoTag = typing.TypedDict('_CiftiParcellateExcludeOutliersParamsDictNoTag', {
    "sigma-below": float,
    "sigma-above": float,
})
CiftiParcellateExcludeOutliersParamsDictTagged = typing.TypedDict('CiftiParcellateExcludeOutliersParamsDictTagged', {
    "@type": typing.Literal["exclude-outliers"],
    "sigma-below": float,
    "sigma-above": float,
})
CiftiParcellateExcludeOutliersParamsDict = _CiftiParcellateExcludeOutliersParamsDictNoTag | CiftiParcellateExcludeOutliersParamsDictTagged


_CiftiParcellateNonemptyMaskOutParamsDictNoTag = typing.TypedDict('_CiftiParcellateNonemptyMaskOutParamsDictNoTag', {
    "mask-out": str,
})
CiftiParcellateNonemptyMaskOutParamsDictTagged = typing.TypedDict('CiftiParcellateNonemptyMaskOutParamsDictTagged', {
    "@type": typing.Literal["nonempty-mask-out"],
    "mask-out": str,
})
CiftiParcellateNonemptyMaskOutParamsDict = _CiftiParcellateNonemptyMaskOutParamsDictNoTag | CiftiParcellateNonemptyMaskOutParamsDictTagged


_CiftiParcellateParamsDictNoTag = typing.TypedDict('_CiftiParcellateParamsDictNoTag', {
    "cifti-in": InputPathType,
    "cifti-label": InputPathType,
    "direction": str,
    "cifti-out": str,
    "spatial-weights": typing.NotRequired[CiftiParcellateSpatialWeightsParamsDict | None],
    "exclude-outliers": typing.NotRequired[CiftiParcellateExcludeOutliersParamsDict | None],
    "nonempty-mask-out": typing.NotRequired[CiftiParcellateNonemptyMaskOutParamsDict | None],
    "value": typing.NotRequired[float | None],
    "method": typing.NotRequired[str | None],
    "weight-cifti": typing.NotRequired[InputPathType | None],
    "include-empty": bool,
    "legacy-mode": bool,
    "only-numeric": bool,
})
CiftiParcellateParamsDictTagged = typing.TypedDict('CiftiParcellateParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-parcellate"],
    "cifti-in": InputPathType,
    "cifti-label": InputPathType,
    "direction": str,
    "cifti-out": str,
    "spatial-weights": typing.NotRequired[CiftiParcellateSpatialWeightsParamsDict | None],
    "exclude-outliers": typing.NotRequired[CiftiParcellateExcludeOutliersParamsDict | None],
    "nonempty-mask-out": typing.NotRequired[CiftiParcellateNonemptyMaskOutParamsDict | None],
    "value": typing.NotRequired[float | None],
    "method": typing.NotRequired[str | None],
    "weight-cifti": typing.NotRequired[InputPathType | None],
    "include-empty": bool,
    "legacy-mode": bool,
    "only-numeric": bool,
})
CiftiParcellateParamsDict = _CiftiParcellateParamsDictNoTag | CiftiParcellateParamsDictTagged


def cifti_parcellate_spatial_weights(
    cerebellum_metric: InputPathType | None = None,
    right_metric: InputPathType | None = None,
    left_metric: InputPathType | None = None,
    cerebellum_surf: InputPathType | None = None,
    right_surf: InputPathType | None = None,
    left_surf: InputPathType | None = None,
) -> CiftiParcellateSpatialWeightsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cerebellum_metric: use a metric file for cerebellum vertex weights\
            \
            metric file containing cerebellum vertex weights.
        right_metric: use a metric file for right vertex weights\
            \
            metric file containing right vertex weights.
        left_metric: use a metric file for left vertex weights\
            \
            metric file containing left vertex weights.
        cerebellum_surf: use a surface for cerebellum vertex areas\
            \
            the cerebellum surface to use, areas are in mm^2.
        right_surf: use a surface for right vertex areas\
            \
            the right surface to use, areas are in mm^2.
        left_surf: use a surface for left vertex areas\
            \
            the left surface to use, areas are in mm^2.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "spatial-weights",
    }
    if cerebellum_metric is not None:
        params["cerebellum-metric"] = cerebellum_metric
    if right_metric is not None:
        params["right-metric"] = right_metric
    if left_metric is not None:
        params["left-metric"] = left_metric
    if cerebellum_surf is not None:
        params["cerebellum-surf"] = cerebellum_surf
    if right_surf is not None:
        params["right-surf"] = right_surf
    if left_surf is not None:
        params["left-surf"] = left_surf
    return params


def cifti_parcellate_spatial_weights_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiParcellateSpatialWeightsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cerebellum-metric", None) is not None:
        if not isinstance(params["cerebellum-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`cerebellum-metric` has the wrong type: Received `{type(params.get("cerebellum-metric", None))}` expected `InputPathType | None`')
    if params.get("right-metric", None) is not None:
        if not isinstance(params["right-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`right-metric` has the wrong type: Received `{type(params.get("right-metric", None))}` expected `InputPathType | None`')
    if params.get("left-metric", None) is not None:
        if not isinstance(params["left-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`left-metric` has the wrong type: Received `{type(params.get("left-metric", None))}` expected `InputPathType | None`')
    if params.get("cerebellum-surf", None) is not None:
        if not isinstance(params["cerebellum-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`cerebellum-surf` has the wrong type: Received `{type(params.get("cerebellum-surf", None))}` expected `InputPathType | None`')
    if params.get("right-surf", None) is not None:
        if not isinstance(params["right-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`right-surf` has the wrong type: Received `{type(params.get("right-surf", None))}` expected `InputPathType | None`')
    if params.get("left-surf", None) is not None:
        if not isinstance(params["left-surf"], (pathlib.Path, str)):
            raise StyxValidationError(f'`left-surf` has the wrong type: Received `{type(params.get("left-surf", None))}` expected `InputPathType | None`')


def cifti_parcellate_spatial_weights_cargs(
    params: CiftiParcellateSpatialWeightsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-spatial-weights")
    if params.get("cerebellum-metric", None) is not None:
        cargs.extend([
            "-cerebellum-area-metric",
            execution.input_file(params.get("cerebellum-metric", None))
        ])
    if params.get("right-metric", None) is not None:
        cargs.extend([
            "-right-area-metric",
            execution.input_file(params.get("right-metric", None))
        ])
    if params.get("left-metric", None) is not None:
        cargs.extend([
            "-left-area-metric",
            execution.input_file(params.get("left-metric", None))
        ])
    if params.get("cerebellum-surf", None) is not None:
        cargs.extend([
            "-cerebellum-area-surf",
            execution.input_file(params.get("cerebellum-surf", None))
        ])
    if params.get("right-surf", None) is not None:
        cargs.extend([
            "-right-area-surf",
            execution.input_file(params.get("right-surf", None))
        ])
    if params.get("left-surf", None) is not None:
        cargs.extend([
            "-left-area-surf",
            execution.input_file(params.get("left-surf", None))
        ])
    return cargs


def cifti_parcellate_exclude_outliers(
    sigma_below: float,
    sigma_above: float,
) -> CiftiParcellateExcludeOutliersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sigma_below: number of standard deviations below the mean to include.
        sigma_above: number of standard deviations above the mean to include.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exclude-outliers",
        "sigma-below": sigma_below,
        "sigma-above": sigma_above,
    }
    return params


def cifti_parcellate_exclude_outliers_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiParcellateExcludeOutliersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("sigma-below", None) is None:
        raise StyxValidationError("`sigma-below` must not be None")
    if not isinstance(params["sigma-below"], (float, int)):
        raise StyxValidationError(f'`sigma-below` has the wrong type: Received `{type(params.get("sigma-below", None))}` expected `float`')
    if params.get("sigma-above", None) is None:
        raise StyxValidationError("`sigma-above` must not be None")
    if not isinstance(params["sigma-above"], (float, int)):
        raise StyxValidationError(f'`sigma-above` has the wrong type: Received `{type(params.get("sigma-above", None))}` expected `float`')


def cifti_parcellate_exclude_outliers_cargs(
    params: CiftiParcellateExcludeOutliersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-exclude-outliers",
        str(params.get("sigma-below", None)),
        str(params.get("sigma-above", None))
    ])
    return cargs


class CiftiParcellateNonemptyMaskOutOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiParcellateNonemptyMaskOutParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    mask_out: OutputPathType
    """the output mask file"""


def cifti_parcellate_nonempty_mask_out(
    mask_out: str,
) -> CiftiParcellateNonemptyMaskOutParamsDictTagged:
    """
    Build parameters.
    
    Args:
        mask_out: the output mask file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "nonempty-mask-out",
        "mask-out": mask_out,
    }
    return params


def cifti_parcellate_nonempty_mask_out_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiParcellateNonemptyMaskOutParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask-out", None) is None:
        raise StyxValidationError("`mask-out` must not be None")
    if not isinstance(params["mask-out"], str):
        raise StyxValidationError(f'`mask-out` has the wrong type: Received `{type(params.get("mask-out", None))}` expected `str`')


def cifti_parcellate_nonempty_mask_out_cargs(
    params: CiftiParcellateNonemptyMaskOutParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-nonempty-mask-out",
        params.get("mask-out", None)
    ])
    return cargs


def cifti_parcellate_nonempty_mask_out_outputs(
    params: CiftiParcellateNonemptyMaskOutParamsDict,
    execution: Execution,
) -> CiftiParcellateNonemptyMaskOutOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiParcellateNonemptyMaskOutOutputs(
        root=execution.output_file("."),
        mask_out=execution.output_file(params.get("mask-out", None)),
    )
    return ret


class CiftiParcellateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiParcellateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""
    nonempty_mask_out: CiftiParcellateNonemptyMaskOutOutputs | None
    """Outputs from `cifti_parcellate_nonempty_mask_out_outputs`."""


def cifti_parcellate_params(
    cifti_in: InputPathType,
    cifti_label: InputPathType,
    direction: str,
    cifti_out: str,
    spatial_weights: CiftiParcellateSpatialWeightsParamsDict | None = None,
    exclude_outliers: CiftiParcellateExcludeOutliersParamsDict | None = None,
    nonempty_mask_out: CiftiParcellateNonemptyMaskOutParamsDict | None = None,
    value: float | None = None,
    method: str | None = None,
    weight_cifti: InputPathType | None = None,
    include_empty: bool = False,
    legacy_mode: bool = False,
    only_numeric: bool = False,
) -> CiftiParcellateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to parcellate.
        cifti_label: a cifti label file to use for the parcellation.
        direction: which mapping to parcellate (integer, ROW, or COLUMN).
        cifti_out: output cifti file.
        spatial_weights: use voxel volume and either vertex areas or metric\
            files as weights.
        exclude_outliers: exclude non-numeric values and outliers from each\
            parcel by standard deviation.
        nonempty_mask_out: output a matching pscalar file that has 0s in empty\
            parcels, and 1s elsewhere.
        value: specify value to use in empty parcels (default 0)\
            \
            the value to fill empty parcels with.
        method: specify method of parcellation (default MEAN, or MODE if label\
            data)\
            \
            the method to use to assign parcel values from the values of member\
            brainordinates.
        weight_cifti: use a cifti file containing weights\
            \
            the weights to use, as a cifti file.
        include_empty: deprecated: now the default behavior.
        legacy_mode: use the old behavior, parcels are defined by the\
            intersection between labels and valid data, and empty parcels are\
            discarded.
        only_numeric: exclude non-numeric values.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-parcellate",
        "cifti-in": cifti_in,
        "cifti-label": cifti_label,
        "direction": direction,
        "cifti-out": cifti_out,
        "include-empty": include_empty,
        "legacy-mode": legacy_mode,
        "only-numeric": only_numeric,
    }
    if spatial_weights is not None:
        params["spatial-weights"] = spatial_weights
    if exclude_outliers is not None:
        params["exclude-outliers"] = exclude_outliers
    if nonempty_mask_out is not None:
        params["nonempty-mask-out"] = nonempty_mask_out
    if value is not None:
        params["value"] = value
    if method is not None:
        params["method"] = method
    if weight_cifti is not None:
        params["weight-cifti"] = weight_cifti
    return params


def cifti_parcellate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiParcellateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("cifti-label", None) is None:
        raise StyxValidationError("`cifti-label` must not be None")
    if not isinstance(params["cifti-label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-label` has the wrong type: Received `{type(params.get("cifti-label", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("spatial-weights", None) is not None:
        cifti_parcellate_spatial_weights_validate(params["spatial-weights"])
    if params.get("exclude-outliers", None) is not None:
        cifti_parcellate_exclude_outliers_validate(params["exclude-outliers"])
    if params.get("nonempty-mask-out", None) is not None:
        cifti_parcellate_nonempty_mask_out_validate(params["nonempty-mask-out"])
    if params.get("value", None) is not None:
        if not isinstance(params["value"], (float, int)):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float | None`')
    if params.get("method", None) is not None:
        if not isinstance(params["method"], str):
            raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str | None`')
    if params.get("weight-cifti", None) is not None:
        if not isinstance(params["weight-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`weight-cifti` has the wrong type: Received `{type(params.get("weight-cifti", None))}` expected `InputPathType | None`')
    if params.get("include-empty", False) is None:
        raise StyxValidationError("`include-empty` must not be None")
    if not isinstance(params["include-empty"], bool):
        raise StyxValidationError(f'`include-empty` has the wrong type: Received `{type(params.get("include-empty", False))}` expected `bool`')
    if params.get("legacy-mode", False) is None:
        raise StyxValidationError("`legacy-mode` must not be None")
    if not isinstance(params["legacy-mode"], bool):
        raise StyxValidationError(f'`legacy-mode` has the wrong type: Received `{type(params.get("legacy-mode", False))}` expected `bool`')
    if params.get("only-numeric", False) is None:
        raise StyxValidationError("`only-numeric` must not be None")
    if not isinstance(params["only-numeric"], bool):
        raise StyxValidationError(f'`only-numeric` has the wrong type: Received `{type(params.get("only-numeric", False))}` expected `bool`')


def cifti_parcellate_cargs(
    params: CiftiParcellateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-parcellate"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(execution.input_file(params.get("cifti-label", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("spatial-weights", None) is not None or params.get("exclude-outliers", None) is not None or params.get("nonempty-mask-out", None) is not None:
        cargs.extend([
            *(cifti_parcellate_spatial_weights_cargs(params.get("spatial-weights", None), execution) if (params.get("spatial-weights", None) is not None) else []),
            *(cifti_parcellate_exclude_outliers_cargs(params.get("exclude-outliers", None), execution) if (params.get("exclude-outliers", None) is not None) else []),
            *(cifti_parcellate_nonempty_mask_out_cargs(params.get("nonempty-mask-out", None), execution) if (params.get("nonempty-mask-out", None) is not None) else [])
        ])
    if params.get("value", None) is not None:
        cargs.extend([
            "-fill-value",
            str(params.get("value", None))
        ])
    if params.get("method", None) is not None:
        cargs.extend([
            "-method",
            params.get("method", None)
        ])
    if params.get("weight-cifti", None) is not None:
        cargs.extend([
            "-cifti-weights",
            execution.input_file(params.get("weight-cifti", None))
        ])
    if params.get("include-empty", False):
        cargs.append("-include-empty")
    if params.get("legacy-mode", False):
        cargs.append("-legacy-mode")
    if params.get("only-numeric", False):
        cargs.append("-only-numeric")
    return cargs


def cifti_parcellate_outputs(
    params: CiftiParcellateParamsDict,
    execution: Execution,
) -> CiftiParcellateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiParcellateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
        nonempty_mask_out=cifti_parcellate_nonempty_mask_out_outputs(params.get("nonempty-mask-out"), execution) if params.get("nonempty-mask-out") else None,
    )
    return ret


def cifti_parcellate_execute(
    params: CiftiParcellateParamsDict,
    runner: Runner | None = None,
) -> CiftiParcellateOutputs:
    """
    PARCELLATE A CIFTI FILE.
    
    Each label (other than the unlabeled key) in the cifti label file will be
    treated as a parcel, and all rows or columns of data within the parcel are
    averaged together to form the parcel's output row or column. If -legacy-mode
    is specified, parcels will be defined as the overlap between a label and the
    data, with no errors for missing data vertices or voxels, and empty parcels
    discarded. The direction can be either an integer starting from 1, or the
    strings 'ROW' or 'COLUMN'. For dtseries or dscalar, use COLUMN. If you are
    parcellating a dconn in both directions, parcellating by ROW first will use
    much less memory.
    
    NOTE: the parcels in the output file are sorted by the numeric label keys,
    in ascending order.
    
    The parameter to the -method option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    
    The -*-weights options are mutually exclusive and may only be used with MEAN
    (default), SUM, STDEV, SAMPSTDEV, VARIANCE, MEDIAN, or MODE (default for
    label data).
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcellateOutputs`).
    """
    cifti_parcellate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_PARCELLATE_METADATA)
    params = execution.params(params)
    cargs = cifti_parcellate_cargs(params, execution)
    ret = cifti_parcellate_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_parcellate(
    cifti_in: InputPathType,
    cifti_label: InputPathType,
    direction: str,
    cifti_out: str,
    spatial_weights: CiftiParcellateSpatialWeightsParamsDict | None = None,
    exclude_outliers: CiftiParcellateExcludeOutliersParamsDict | None = None,
    nonempty_mask_out: CiftiParcellateNonemptyMaskOutParamsDict | None = None,
    value: float | None = None,
    method: str | None = None,
    weight_cifti: InputPathType | None = None,
    include_empty: bool = False,
    legacy_mode: bool = False,
    only_numeric: bool = False,
    runner: Runner | None = None,
) -> CiftiParcellateOutputs:
    """
    PARCELLATE A CIFTI FILE.
    
    Each label (other than the unlabeled key) in the cifti label file will be
    treated as a parcel, and all rows or columns of data within the parcel are
    averaged together to form the parcel's output row or column. If -legacy-mode
    is specified, parcels will be defined as the overlap between a label and the
    data, with no errors for missing data vertices or voxels, and empty parcels
    discarded. The direction can be either an integer starting from 1, or the
    strings 'ROW' or 'COLUMN'. For dtseries or dscalar, use COLUMN. If you are
    parcellating a dconn in both directions, parcellating by ROW first will use
    much less memory.
    
    NOTE: the parcels in the output file are sorted by the numeric label keys,
    in ascending order.
    
    The parameter to the -method option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    
    The -*-weights options are mutually exclusive and may only be used with MEAN
    (default), SUM, STDEV, SAMPSTDEV, VARIANCE, MEDIAN, or MODE (default for
    label data).
    
    Args:
        cifti_in: the cifti file to parcellate.
        cifti_label: a cifti label file to use for the parcellation.
        direction: which mapping to parcellate (integer, ROW, or COLUMN).
        cifti_out: output cifti file.
        spatial_weights: use voxel volume and either vertex areas or metric\
            files as weights.
        exclude_outliers: exclude non-numeric values and outliers from each\
            parcel by standard deviation.
        nonempty_mask_out: output a matching pscalar file that has 0s in empty\
            parcels, and 1s elsewhere.
        value: specify value to use in empty parcels (default 0)\
            \
            the value to fill empty parcels with.
        method: specify method of parcellation (default MEAN, or MODE if label\
            data)\
            \
            the method to use to assign parcel values from the values of member\
            brainordinates.
        weight_cifti: use a cifti file containing weights\
            \
            the weights to use, as a cifti file.
        include_empty: deprecated: now the default behavior.
        legacy_mode: use the old behavior, parcels are defined by the\
            intersection between labels and valid data, and empty parcels are\
            discarded.
        only_numeric: exclude non-numeric values.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiParcellateOutputs`).
    """
    params = cifti_parcellate_params(
        cifti_in=cifti_in,
        cifti_label=cifti_label,
        direction=direction,
        cifti_out=cifti_out,
        spatial_weights=spatial_weights,
        exclude_outliers=exclude_outliers,
        nonempty_mask_out=nonempty_mask_out,
        value=value,
        method=method,
        weight_cifti=weight_cifti,
        include_empty=include_empty,
        legacy_mode=legacy_mode,
        only_numeric=only_numeric,
    )
    return cifti_parcellate_execute(params, runner)


__all__ = [
    "CIFTI_PARCELLATE_METADATA",
    "CiftiParcellateExcludeOutliersParamsDict",
    "CiftiParcellateExcludeOutliersParamsDictTagged",
    "CiftiParcellateNonemptyMaskOutOutputs",
    "CiftiParcellateNonemptyMaskOutParamsDict",
    "CiftiParcellateNonemptyMaskOutParamsDictTagged",
    "CiftiParcellateOutputs",
    "CiftiParcellateParamsDict",
    "CiftiParcellateParamsDictTagged",
    "CiftiParcellateSpatialWeightsParamsDict",
    "CiftiParcellateSpatialWeightsParamsDictTagged",
    "cifti_parcellate",
    "cifti_parcellate_exclude_outliers",
    "cifti_parcellate_execute",
    "cifti_parcellate_nonempty_mask_out",
    "cifti_parcellate_params",
    "cifti_parcellate_spatial_weights",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_PARCEL_RESAMPLING_METADATA = Metadata(
    id="b8666fd0e3fda0690696a4034402cad85826cbf3.workbench",
    name="volume-parcel-resampling",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeParcelResamplingParamsDictNoTag = typing.TypedDict('_VolumeParcelResamplingParamsDictNoTag', {
    "volume-in": InputPathType,
    "cur-parcels": InputPathType,
    "new-parcels": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "fwhm": bool,
    "fix-zeros": bool,
})
VolumeParcelResamplingParamsDictTagged = typing.TypedDict('VolumeParcelResamplingParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-parcel-resampling"],
    "volume-in": InputPathType,
    "cur-parcels": InputPathType,
    "new-parcels": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "fwhm": bool,
    "fix-zeros": bool,
})
VolumeParcelResamplingParamsDict = _VolumeParcelResamplingParamsDictNoTag | VolumeParcelResamplingParamsDictTagged


class VolumeParcelResamplingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeParcelResamplingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """output volume"""


def volume_parcel_resampling_params(
    volume_in: InputPathType,
    cur_parcels: InputPathType,
    new_parcels: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    fwhm: bool = False,
    fix_zeros: bool = False,
) -> VolumeParcelResamplingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input data volume.
        cur_parcels: label volume of where the parcels currently are.
        new_parcels: label volume of where the parcels should be.
        kernel: gaussian kernel size in mm to smooth by during resampling, as\
            sigma by default.
        volume_out: output volume.
        subvol: select a single subvolume as input\
            \
            the subvolume number or name.
        fwhm: smoothing kernel size is FWHM, not sigma.
        fix_zeros: treat zero values as not being data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-parcel-resampling",
        "volume-in": volume_in,
        "cur-parcels": cur_parcels,
        "new-parcels": new_parcels,
        "kernel": kernel,
        "volume-out": volume_out,
        "fwhm": fwhm,
        "fix-zeros": fix_zeros,
    }
    if subvol is not None:
        params["subvol"] = subvol
    return params


def volume_parcel_resampling_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeParcelResamplingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("cur-parcels", None) is None:
        raise StyxValidationError("`cur-parcels` must not be None")
    if not isinstance(params["cur-parcels"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cur-parcels` has the wrong type: Received `{type(params.get("cur-parcels", None))}` expected `InputPathType`')
    if params.get("new-parcels", None) is None:
        raise StyxValidationError("`new-parcels` must not be None")
    if not isinstance(params["new-parcels"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-parcels` has the wrong type: Received `{type(params.get("new-parcels", None))}` expected `InputPathType`')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')
    if params.get("fix-zeros", False) is None:
        raise StyxValidationError("`fix-zeros` must not be None")
    if not isinstance(params["fix-zeros"], bool):
        raise StyxValidationError(f'`fix-zeros` has the wrong type: Received `{type(params.get("fix-zeros", False))}` expected `bool`')


def volume_parcel_resampling_cargs(
    params: VolumeParcelResamplingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-parcel-resampling"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(execution.input_file(params.get("cur-parcels", None)))
    cargs.append(execution.input_file(params.get("new-parcels", None)))
    cargs.append(str(params.get("kernel", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    if params.get("fix-zeros", False):
        cargs.append("-fix-zeros")
    return cargs


def volume_parcel_resampling_outputs(
    params: VolumeParcelResamplingParamsDict,
    execution: Execution,
) -> VolumeParcelResamplingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeParcelResamplingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_parcel_resampling_execute(
    params: VolumeParcelResamplingParamsDict,
    runner: Runner | None = None,
) -> VolumeParcelResamplingOutputs:
    """
    SMOOTH AND RESAMPLE VOLUME PARCELS.
    
    Smooths and resamples the region inside each label in cur-parcels to the
    region of the same label name in new-parcels. Any voxels in the output label
    region but outside the input label region will be extrapolated from nearby
    data. The -fix-zeros option causes the smoothing to not use an input value
    if it is zero, but still write a smoothed value to the voxel, and after
    smoothing is complete, it will check for any remaining values of zero, and
    fill them in with extrapolated values.
    
    Note: all volumes must have the same dimensions and spacing. To use a
    different output space, see -volume-parcel-resampling-generic.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelResamplingOutputs`).
    """
    volume_parcel_resampling_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_PARCEL_RESAMPLING_METADATA)
    params = execution.params(params)
    cargs = volume_parcel_resampling_cargs(params, execution)
    ret = volume_parcel_resampling_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_parcel_resampling(
    volume_in: InputPathType,
    cur_parcels: InputPathType,
    new_parcels: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    fwhm: bool = False,
    fix_zeros: bool = False,
    runner: Runner | None = None,
) -> VolumeParcelResamplingOutputs:
    """
    SMOOTH AND RESAMPLE VOLUME PARCELS.
    
    Smooths and resamples the region inside each label in cur-parcels to the
    region of the same label name in new-parcels. Any voxels in the output label
    region but outside the input label region will be extrapolated from nearby
    data. The -fix-zeros option causes the smoothing to not use an input value
    if it is zero, but still write a smoothed value to the voxel, and after
    smoothing is complete, it will check for any remaining values of zero, and
    fill them in with extrapolated values.
    
    Note: all volumes must have the same dimensions and spacing. To use a
    different output space, see -volume-parcel-resampling-generic.
    
    Args:
        volume_in: the input data volume.
        cur_parcels: label volume of where the parcels currently are.
        new_parcels: label volume of where the parcels should be.
        kernel: gaussian kernel size in mm to smooth by during resampling, as\
            sigma by default.
        volume_out: output volume.
        subvol: select a single subvolume as input\
            \
            the subvolume number or name.
        fwhm: smoothing kernel size is FWHM, not sigma.
        fix_zeros: treat zero values as not being data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeParcelResamplingOutputs`).
    """
    params = volume_parcel_resampling_params(
        volume_in=volume_in,
        cur_parcels=cur_parcels,
        new_parcels=new_parcels,
        kernel=kernel,
        volume_out=volume_out,
        subvol=subvol,
        fwhm=fwhm,
        fix_zeros=fix_zeros,
    )
    return volume_parcel_resampling_execute(params, runner)


__all__ = [
    "VOLUME_PARCEL_RESAMPLING_METADATA",
    "VolumeParcelResamplingOutputs",
    "VolumeParcelResamplingParamsDict",
    "VolumeParcelResamplingParamsDictTagged",
    "volume_parcel_resampling",
    "volume_parcel_resampling_execute",
    "volume_parcel_resampling_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_ESTIMATE_FWHM_METADATA = Metadata(
    id="9c0ae9d2b6ef7d3111a63f811feffe7ae9dde13f.workbench",
    name="metric-estimate-fwhm",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricEstimateFwhmParamsDictNoTag = typing.TypedDict('_MetricEstimateFwhmParamsDictNoTag', {
    "surface": InputPathType,
    "metric-in": InputPathType,
    "demean": typing.NotRequired[bool | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricEstimateFwhmParamsDictTagged = typing.TypedDict('MetricEstimateFwhmParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-estimate-fwhm"],
    "surface": InputPathType,
    "metric-in": InputPathType,
    "demean": typing.NotRequired[bool | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricEstimateFwhmParamsDict = _MetricEstimateFwhmParamsDictNoTag | MetricEstimateFwhmParamsDictTagged


class MetricEstimateFwhmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricEstimateFwhmParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metric_estimate_fwhm_params(
    surface: InputPathType,
    metric_in: InputPathType,
    demean: bool | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
) -> MetricEstimateFwhmParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for distance and neighbor information.
        metric_in: the input metric.
        demean: estimate for the whole file at once, not each column separately\
            \
            subtract the mean image before estimating smoothness.
        column: select a single column to estimate smoothness of\
            \
            the column number or name.
        roi_metric: use only data within an ROI\
            \
            the metric file to use as an ROI.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-estimate-fwhm",
        "surface": surface,
        "metric-in": metric_in,
    }
    if demean is not None:
        params["demean"] = demean
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_estimate_fwhm_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricEstimateFwhmParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("demean", None) is not None:
        if not isinstance(params["demean"], bool):
            raise StyxValidationError(f'`demean` has the wrong type: Received `{type(params.get("demean", None))}` expected `bool | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def metric_estimate_fwhm_cargs(
    params: MetricEstimateFwhmParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-estimate-fwhm"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric-in", None)))
    if params.get("demean", None) is not None:
        cargs.extend([
            "-whole-file",
            "-demean"
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def metric_estimate_fwhm_outputs(
    params: MetricEstimateFwhmParamsDict,
    execution: Execution,
) -> MetricEstimateFwhmOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricEstimateFwhmOutputs(
        root=execution.output_file("."),
    )
    return ret


def metric_estimate_fwhm_execute(
    params: MetricEstimateFwhmParamsDict,
    runner: Runner | None = None,
) -> MetricEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A METRIC FILE.
    
    Estimates the smoothness of the metric columns, printing the estimates to
    standard output. These estimates ignore variation in vertex spacing.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricEstimateFwhmOutputs`).
    """
    metric_estimate_fwhm_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_ESTIMATE_FWHM_METADATA)
    params = execution.params(params)
    cargs = metric_estimate_fwhm_cargs(params, execution)
    ret = metric_estimate_fwhm_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_estimate_fwhm(
    surface: InputPathType,
    metric_in: InputPathType,
    demean: bool | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A METRIC FILE.
    
    Estimates the smoothness of the metric columns, printing the estimates to
    standard output. These estimates ignore variation in vertex spacing.
    
    Args:
        surface: the surface to use for distance and neighbor information.
        metric_in: the input metric.
        demean: estimate for the whole file at once, not each column separately\
            \
            subtract the mean image before estimating smoothness.
        column: select a single column to estimate smoothness of\
            \
            the column number or name.
        roi_metric: use only data within an ROI\
            \
            the metric file to use as an ROI.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricEstimateFwhmOutputs`).
    """
    params = metric_estimate_fwhm_params(
        surface=surface,
        metric_in=metric_in,
        demean=demean,
        column=column,
        roi_metric=roi_metric,
    )
    return metric_estimate_fwhm_execute(params, runner)


__all__ = [
    "METRIC_ESTIMATE_FWHM_METADATA",
    "MetricEstimateFwhmOutputs",
    "MetricEstimateFwhmParamsDict",
    "MetricEstimateFwhmParamsDictTagged",
    "metric_estimate_fwhm",
    "metric_estimate_fwhm_execute",
    "metric_estimate_fwhm_params",
]
